#include "utils.hpp"
#include <libscerror.hpp>
#include <cmath>
#include <iostream>
#include <iomanip>
#include <string>
#include <stdexcept>
#include "iomanip.hpp"

using namespace sc;

term::term(int fd) {
    if (isatty(fd)) {
        throw_if_min1(ioctl(fd, TIOCGWINSZ, &_ws));
    }
    switch (fd) {
        case STDOUT_FILENO: _out = &std::cout; break;
        case STDERR_FILENO: _out = &std::cerr; break;
        default: break;
    }
}

int term::rows() const {
    return _ws.ws_row;
}

int term::cols() const {
    return _ws.ws_col;
}

void term::progress(int prefixlen,
                    const std::string& prefix,
                    double cur,
                    double total) const
{
    // use unicode to make nice bar
    const char* bar[8] = {
        u8"\u258F", u8"\u258E", u8"\u258D", u8"\u258C",
        u8"\u258B", u8"\u258A", u8"\u2589", u8"\u2588"
    };
    int perclen = 5;
    int barwidth = cols() - perclen - prefixlen - 1;
    int maxsteps = barwidth * 8;
    int steps = round(cur * maxsteps / total);
    int perc = round(100 * cur / total);
    int fill_len = barwidth - steps / 8;
    *_out << '\r' << std::setw(prefixlen) << prefix.substr(0, prefixlen) << ' ';
    grayf(8);
    for (int i = 0; i < steps / 8; ++i) {
        *_out << bar[7];
    }
    if (steps % 8) {
        *_out << bar[steps % 8 - 1];
        fill_len--;
    }
    *_out << io::reset << std::string(fill_len, ' ');
    *_out << ' ' << std::setw(3) << perc << '%';
}

void term::hide_cursor() const {
    *_out << io::hide_cursor;
}

void term::show_cursor() const {
    *_out << io::show_cursor;
}

void term::reset() const {
    *_out << io::reset;
}

void term::bold() const {
    *_out << io::bold;
}

void term::italic() const {
    *_out << io::italic;
}

void term::underline() const {
    *_out << io::underline;
}

void term::overline() const {
    *_out << io::overline;
}

void term::strikethru() const {
    *_out << io::strikethru;
}

void term::blinkslow() const {
    *_out << io::blinkslow;
}

void term::blinkfast() const {
    *_out << io::blinkfast;
}

void term::reverse() const {
    *_out << io::reverse;
}

void term::blackf() const {
    *_out << io::blackf;
}

void term::redf() const {
    *_out << io::redf;
}

void term::greenf() const {
    *_out << io::greenf;
}

void term::yellowf() const {
    *_out << io::yellowf;
}

void term::bluef() const {
    *_out << io::bluef;
}

void term::magentaf() const {
    *_out << io::magentaf;
}

void term::cyanf() const {
    *_out << io::cyanf;
}

void term::whitef() const {
    *_out << io::whitef;
}

void term::defaultf() const {
    *_out << io::defaultf;
}

void term::grayf(int val) const {
    *_out << io::grayf(val);
}

void term::rgbf(int r, int g, int b) const {
    *_out << io::rgbf(r, g, b);
}

void term::blackb() const {
    *_out << io::blackb;
}

void term::redb() const {
    *_out << io::redb;
}

void term::greenb() const {
    *_out << io::greenb;
}

void term::yellowb() const {
    *_out << io::yellowb;
}

void term::blueb() const {
    *_out << io::blueb;
}

void term::magentab() const {
    *_out << io::magentab;
}

void term::cyanb() const {
    *_out << io::cyanb;
}

void term::whiteb() const {
    *_out << io::whiteb;
}

void term::defaultb() const {
    *_out << io::defaultb;
}

void term::grayb(int val) const {
    *_out << io::grayb(val);
}

void term::rgbb(int r, int g, int b) const {
    *_out << io::rgbb(r, g, b);
}

// ------------------------------------------------------------------------

cursor_hider::cursor_hider(int fd) : _t {fd} {
    _t.hide_cursor();
}

cursor_hider::~cursor_hider() {
    _t.show_cursor();
}
