#ifndef UTILS_H_
#define UTILS_H_

#include <unistd.h>
#include <termios.h>
#include <sys/ioctl.h>
#include <iostream>
#include <string>

namespace sc {

    class term {
        struct winsize _ws;
        std::ostream* _out {nullptr};

        public:
            term(int fd);

            int rows() const;
            int cols() const;

            void progress(int prefixlen,
                          const std::string& prefix,
                          double cur,
                          double total) const;

            void hide_cursor() const;
            void show_cursor() const;

            void reset() const;

            void bold() const;
            void italic() const;
            void underline() const;
            void overline() const;
            void strikethru() const;
            void blinkslow() const;
            void blinkfast() const;
            void reverse() const;

            void blackf() const;
            void redf() const;
            void greenf() const;
            void yellowf() const;
            void bluef() const;
            void magentaf() const;
            void cyanf() const;
            void whitef() const;
            void defaultf() const;

            void grayf(int val) const; // val = 0..23
            void rgbf(int r, int g, int b) const; // r,g,b = 0..5

            void blackb() const;
            void redb() const;
            void greenb() const;
            void yellowb() const;
            void blueb() const;
            void magentab() const;
            void cyanb() const;
            void whiteb() const;
            void defaultb() const;

            void grayb(int val) const; // val = 0..23
            void rgbb(int r, int g, int b) const; // r,g,b = 0..5
    };

    class cursor_hider {
        term _t;

        public:
            cursor_hider(int fd);
            ~cursor_hider();
    };

}

#endif // UTILS_H_
